<?php

/**
 * @file
 * Tests for plugins/FeedsNodeProcessor.inc.
 */

namespace Drupal\feeds\Tests;

use Drupal\feeds\Plugin\FeedsConfigurable;

/**
 * Test aggregating a feed as node items.
 */
class FeedsRSStoNodesTest extends FeedsWebTestBase {

  public static function getInfo() {
    return array(
      'name' => 'RSS import to nodes',
      'description' => 'Tests a feed configuration that is attached to a content type, uses HTTP fetcher, common syndication parser and a node processor. Repeats the same test for an importer configuration that is not attached to a content type and for a configuration that is attached to a content type and uses the file fetcher.',
      'group' => 'Feeds',
    );
  }

  /**
   * Set up test.
   */
  public function setUp() {
    parent::setUp();

    // Set the front page to show 20 nodes so we can easily see what is aggregated.
    variable_set('default_nodes_main', 20);

    // Set the teaser length display to unlimited otherwise tests looking for
    // text on nodes will fail.
    $edit = array('fields[body][type]' => 'text_default');
    $this->drupalPost('admin/structure/types/manage/article/display/teaser', $edit, 'Save');

    // Create an importer configuration.
    FeedsConfigurable::$instances = array();
    $this->createImporterConfiguration('Syndication', 'syndication');
    $this->addMappings('syndication', array(
      0 => array(
        'source' => 'title',
        'target' => 'title',
        'unique' => FALSE,
      ),
      1 => array(
        'source' => 'description',
        'target' => 'body',
      ),
      2 => array(
        'source' => 'timestamp',
        'target' => 'created',
      ),
      3 => array(
        'source' => 'url',
        'target' => 'url',
        'unique' => TRUE,
      ),
      4 => array(
        'source' => 'guid',
        'target' => 'guid',
        'unique' => TRUE,
      ),
    ));
  }

  /**
   * Test node creation, refreshing/deleting feeds and feed items.
   */
  public function test() {
    $nid = $this->createFeedNode();

    // Assert 10 items aggregated after creation of the node.
    $this->assertText('Created 10 nodes');
    $article_nid = db_query_range("SELECT nid FROM {node} WHERE type = 'article'", 0, 1)->fetchField();
    $this->assertEqual("Created by FeedsNodeProcessor", db_query("SELECT nr.log FROM {node} n JOIN {node_revision} nr ON n.vid = nr.vid WHERE n.nid = :nid", array(':nid' => $article_nid))->fetchField());

    // Navigate to feed node, there should be Feeds tabs visible.
    $this->drupalGet("node/$nid");
    $this->assertRaw("node/$nid/import");
    $this->assertRaw("node/$nid/delete-items");

    // Assert accuracy of aggregated information.
    $this->drupalGet('node');
    $this->assertRaw('<span class="username">Anonymous (not verified)</span>');
    $this->assertDevseedFeedContent();

    // Assert DB status.
    $count = db_query("SELECT COUNT(*) FROM {node} n INNER JOIN {feeds_item} fi ON fi.entity_type = 'node' AND n.nid = fi.entity_id")->fetchField();
    $this->assertEqual($count, 10, 'Accurate number of items in database.');

    // Assert default input format on first imported feed node.

    // NEEDS update.
    // $format = db_query_range("SELECT nr.format FROM {feeds_node_item} fi JOIN {node} n ON fi.nid = n.nid JOIN {node_revision} nr ON n.vid = nr.vid", 0, 1)->fetchField();
    // $this->assertEqual($format, filter_fallback_format(), 'Using default Input format.');

    // Import again.
    $this->drupalPost("node/$nid/import", array(), 'Import');
    $this->assertText('There are no new nodes');

    // Assert DB status, there still shouldn't be more than 10 items.
    $count = db_query("SELECT COUNT(*) FROM {node} n INNER JOIN {feeds_item} fi ON fi.entity_type = 'node' AND n.nid = fi.entity_id")->fetchField();
    $this->assertEqual($count, 10, 'Accurate number of items in database.');

    // All of the above tests should have produced published nodes, set default
    // to unpublished, import again.
    $count = db_query("SELECT COUNT(*) FROM {node} n INNER JOIN {feeds_item} fi ON fi.entity_type = 'node' AND n.nid = fi.entity_id WHERE n.status = 1")->fetchField();
    $this->assertEqual($count, 10, 'All items are published.');
    $edit = array(
      'node_options[status]' => FALSE,
    );
    $this->drupalPost('admin/structure/types/manage/article', $edit, t('Save content type'));
    $this->drupalPost("node/$nid/delete-items", array(), 'Delete');
    $this->drupalPost("node/$nid/import", array(), 'Import');
    $count = db_query("SELECT COUNT(*) FROM {node} n INNER JOIN {feeds_item} fi ON fi.entity_type = 'node' AND n.nid = fi.entity_id WHERE n.status = 0")->fetchField();
    $this->assertEqual($count, 10, 'No items are published.');
    $edit = array(
      'node_options[status]' => TRUE,
    );
    $this->drupalPost('admin/structure/types/manage/article', $edit, t('Save content type'));
    $this->drupalPost("node/$nid/delete-items", array(), 'Delete');

    // Enable replace existing and import updated feed file.
    $this->drupalPost("node/$nid/import", array(), 'Import');
    $this->setSettings('syndication', 'node', array('update_existing' => 1));
    $feed_url = $GLOBALS['base_url'] . '/' . drupal_get_path('module', 'feeds') . '/tests/feeds/developmentseed_changes.rss2';
    $this->editFeedNode($nid, $feed_url);
    $this->drupalPost("node/$nid/import", array(), 'Import');
    $this->assertText('Updated 2 nodes');

    // Assert accuracy of aggregated content (check 2 updates, one original).
    $this->drupalGet('node');
    $this->assertText('Managing News Translation Workflow: Two Way Translation Updates');
    $this->assertText('Presenting on Features in Drupal and Managing News');
    $this->assertText('Scaling the Open Atrium UI');

    // Import again.
    $this->drupalPost("node/$nid/import", array(), 'Import');
    $this->assertText('There are no new nodes');
    $this->assertFeedItemCount(10);

    // Now delete all items.
    $this->drupalPost("node/$nid/delete-items", array(), 'Delete');
    $this->assertText('Deleted 10 nodes');
    $this->assertFeedItemCount(0);

    // Change author and turn off authorization.
    $this->auth_user = $this->drupalCreateUser(array('access content'));
    $this->setSettings('syndication', 'node', array('author' => $this->auth_user->name, 'authorize' => FALSE));

    // Change input format.
    $this->setSettings('syndication', 'node', array('input_format' => 'plain_text'));

    // Import again.
    $this->drupalPost("node/$nid/import", array(), 'Import');
    $this->assertText('Created 10 nodes');

    // Assert author.
    $this->drupalGet('node');
    $this->assertPattern('/<span class="username">' . check_plain($this->auth_user->name) . '<\/span>/');
    $count = db_query("SELECT COUNT(*) FROM {feeds_item} fi JOIN {node} n ON fi.entity_type = 'node' AND fi.entity_id = n.nid WHERE n.uid = :uid", array(':uid' => $this->auth_user->uid))->fetchField();
    $this->assertEqual($count, 10, 'Accurate number of items in database.');

    // Assert input format.

    // NEEDS update.
    // $format = db_query_range("SELECT nr.format FROM {feeds_node_item} fi JOIN {node} n ON fi.nid = n.nid JOIN {node_revision} nr ON n.vid = nr.vid", 0, 1)->fetchField();
    // $this->assertEqual($format, filter_fallback_format() + 1, 'Set non-default Input format.');

    // Set to update existing, remove authorship of above nodes and import again.
    $this->setSettings('syndication', 'node', array('update_existing' => 2));
    $nids = db_query("SELECT nid FROM {node} n INNER JOIN {feeds_item} fi ON fi.entity_type = 'node' AND n.nid = fi.entity_id")->fetchCol();
    db_update('node')
      ->fields(array('uid' => 0))
      ->condition('nid', $nids, 'IN')
      ->execute();
    db_update('feeds_item')
      ->fields(array('hash' => ''))
      ->condition('entity_type', 'node')
      ->condition('entity_id', $nids, 'IN')
      ->execute();
    $this->drupalPost("node/$nid/import", array(), 'Import');
    $this->drupalGet('node');
    $this->assertNoPattern('/<span class="username">' . check_plain($this->auth_user->name) . '<\/span>/');
    $count = db_query("SELECT COUNT(*) FROM {feeds_item} fi JOIN {node} n ON fi.entity_type = 'node' AND fi.entity_id = n.nid WHERE n.uid = :uid", array(':uid' => $this->auth_user->uid))->fetchField();
    $this->assertEqual($count, 0, 'Accurate number of items in database.');

    // Map feed node's author to feed item author, update - feed node's items
    // should now be assigned to feed node author.
    $this->addMappings('syndication', array(
      5 => array(
        'source' => 'parent:uid',
        'target' => 'uid',
      ),
    ));
    $this->drupalPost("node/$nid/import", array(), 'Import');
    $this->drupalGet('node');
    $this->assertNoPattern('/<span class="username">' . check_plain($this->auth_user->name) . '<\/span>/');
    $uid = db_query("SELECT uid FROM {node} WHERE nid = :nid", array(':nid' => $nid))->fetchField();
    $count = db_query("SELECT COUNT(*) FROM {node} WHERE uid = :uid", array(':uid' => $uid))->fetchField();
    $this->assertEqual($count, 11, 'All feed item nodes are assigned to feed node author.');

    // Login with new user with only access content permissions.
    $this->drupalLogin($this->auth_user);

    // Navigate to feed node, there should be no Feeds tabs visible.
    $this->drupalGet("node/$nid");
    $this->assertNoRaw("node/$nid/import");
    $this->assertNoRaw("node/$nid/delete-items");

    // Now create a second feed configuration that is not attached to a content
    // type and run tests on importing/purging.

    // Login with sufficient permissions.
    $this->drupalLogin($this->admin_user);
    // Remove all items again so that next test can check for them.
    $this->drupalPost("node/$nid/delete-items", array(), 'Delete');

    // Create an importer, not attached to content type.
    $this->createImporterConfiguration('Syndication standalone', 'syndication_standalone');
    $edit = array(
      'content_type' => '',
    );
    $this->drupalPost('admin/structure/feeds/syndication_standalone/settings', $edit, 'Save');
    $this->addMappings('syndication_standalone', array(
      0 => array(
        'source' => 'title',
        'target' => 'title',
        'unique' => FALSE,
      ),
      1 => array(
        'source' => 'description',
        'target' => 'body',
      ),
      2 => array(
        'source' => 'timestamp',
        'target' => 'created',
      ),
      3 => array(
        'source' => 'url',
        'target' => 'url',
        'unique' => TRUE,
      ),
      4 => array(
        'source' => 'guid',
        'target' => 'guid',
        'unique' => TRUE,
      ),
    ));

    // Import, assert 10 items aggregated after creation of the node.
    $this->importURL('syndication_standalone');
    $this->assertText('Created 10 nodes');

    // Assert accuracy of aggregated information.
    $this->drupalGet('node');
    $this->assertDevseedFeedContent();
    $this->assertFeedItemCount(10);

    // Import again.
    $this->drupalPost('import/syndication_standalone', array(), 'Import');
    $this->assertText('There are no new nodes');
    $this->assertFeedItemCount(10);

    // Enable replace existing and import updated feed file.
    $this->setSettings('syndication_standalone', 'node', array('update_existing' => 1));
    $feed_url = $GLOBALS['base_url'] . '/' . drupal_get_path('module', 'feeds') . '/tests/feeds/developmentseed_changes.rss2';
    $this->importURL('syndication_standalone', $feed_url);
    $this->assertText('Updated 2 nodes');

    // Assert accuracy of aggregated information (check 2 updates, one orig).
    $this->drupalGet('node');
    $this->assertText('Managing News Translation Workflow: Two Way Translation Updates');
    $this->assertText('Presenting on Features in Drupal and Managing News');
    $this->assertText('Scaling the Open Atrium UI');

    // Import again.
    $this->drupalPost('import/syndication_standalone', array(), 'Import');
    $this->assertText('There are no new nodes');
    $this->assertFeedItemCount(10);

    // Now delete all items.
    $this->drupalPost('import/syndication_standalone/delete-items', array(), 'Delete');
    $this->assertText('Deleted 10 nodes');
    $this->assertFeedItemCount(0);

    // Import again, we should find new content.
    $this->drupalPost('import/syndication_standalone', array(), 'Import');
    $this->assertText('Created 10 nodes');
    $this->assertFeedItemCount(10);

    // Login with new user with only access content permissions.
    $this->drupalLogin($this->auth_user);

    // Navigate to feed import form, access should be denied.
    $this->drupalGet('import/syndication_standalone');
    $this->assertResponse(403);

    // Use File Fetcher.
    $this->drupalLogin($this->admin_user);

    $this->setPlugin('syndication_standalone', 'file');
    $this->setSettings('syndication_standalone', 'file', array('allowed_extensions' => 'rss2'));

    // Create a feed node.
    $edit = array(
      'files[feeds]' => $this->absolutePath() . '/tests/feeds/drupalplanet.rss2',
    );
    $this->drupalPost('import/syndication_standalone', $edit, 'Import');
    $this->assertText('Created 25 nodes');
  }

  /**
   * Check that the total number of entries in the feeds_item table is correct.
   */
  function assertFeedItemCount($num) {
    $count = db_query("SELECT COUNT(*) FROM {feeds_item} WHERE entity_type = 'node'")->fetchField();
    $this->assertEqual($count, $num, 'Accurate number of items in database.');
  }

  /**
   * Check thet contents of the current page for the DS feed.
   */
  function assertDevseedFeedContent() {
    $this->assertText('Open Atrium Translation Workflow: Two Way Translation Updates');
    $this->assertText('Tue, 10/06/2009');
    $this->assertText('A new translation process for Open Atrium &amp; integration with Localize Drupal');
    $this->assertText('Week in DC Tech: October 5th Edition');
    $this->assertText('Mon, 10/05/2009');
    $this->assertText('There are some great technology events happening this week');
    $this->assertText('Mapping Innovation at the World Bank with Open Atrium');
    $this->assertText('Fri, 10/02/2009');
    $this->assertText('is being used as a base platform for collaboration at the World Bank');
    $this->assertText('September GeoDC Meetup Tonight');
    $this->assertText('Wed, 09/30/2009');
    $this->assertText('Today is the last Wednesday of the month');
    $this->assertText('Week in DC Tech: September 28th Edition');
    $this->assertText('Mon, 09/28/2009');
    $this->assertText('Looking to geek out this week? There are a bunch of');
    $this->assertText('Open Data for Microfinance: The New MIXMarket.org');
    $this->assertText('Thu, 09/24/2009');
    $this->assertText('There are profiles for every country that the MIX Market is hosting.');
    $this->assertText('Integrating the Siteminder Access System in an Open Atrium-based Intranet');
    $this->assertText('Tue, 09/22/2009');
    $this->assertText('In addition to authentication, the Siteminder system');
    $this->assertText('Week in DC Tech: September 21 Edition');
    $this->assertText('Mon, 09/21/2009');
    $this->assertText('an interesting variety of technology events happening in Washington, DC ');
    $this->assertText('s Software Freedom Day: Impressions &amp; Photos');
    $this->assertText('Mon, 09/21/2009');
    $this->assertText('Presenting on Features in Drupal and Open Atrium');
    $this->assertText('Scaling the Open Atrium UI');
    $this->assertText('Fri, 09/18/2009');
    $this->assertText('The first major change is switching');
  }

  /**
   * Test validation of feed URLs.
   */
  function testFeedURLValidation() {
    $edit = array(
      'feeds[Drupal\feeds\Plugin\feeds\fetcher\FeedsHTTPFetcher][source]' => 'invalid://url',
    );
    $this->drupalPost('node/add/page', $edit, 'Save and publish');
    $this->assertText('The URL invalid://url is invalid.');
  }

  /**
   * Test using non-normal URLs like feed:// and webcal://.
   */
  function testOddFeedSchemes() {
    $url = $GLOBALS['base_url'] . '/' . drupal_get_path('module', 'feeds') . '/tests/feeds/developmentseed.rss2';

    $schemes = array('feed', 'webcal');
    $item_count = 0;
    foreach ($schemes as $scheme) {
      $feed_url = strtr($url, array('http://' => $scheme . '://', 'https://' => $scheme . '://'));

      $edit['feeds[Drupal\feeds\Plugin\feeds\fetcher\FeedsHTTPFetcher][source]'] = $feed_url;
      $this->drupalPost('node/add/page', $edit, 'Save and publish');
      $this->assertText('Basic page Development Seed - Technological Solutions for Progressive Organizations has been created.');
      $this->assertText('Created 10 nodes.');
      $this->assertFeedItemCount($item_count + 10);
      $item_count += 10;
    }
  }

  /**
   * Test that feed elements and links are not found on non-feed nodes.
   */
  function testNonFeedNodeUI() {
    // There should not be feed links on an article node.
    $non_feed_node = $this->drupalCreateNode(array('type' => 'article'));
    $this->drupalGet('node/' . $non_feed_node->nid);
    $this->assertNoLinkByHref('node/' . $non_feed_node->nid . '/import');
    $this->assertNoLink('Delete items');

    // Navigate to a non-feed node form, there should be no Feed field visible.
    $this->drupalGet('node/add/article');
    $this->assertNoFieldByName('feeds[Drupal\feeds\Plugin\feeds\fetcher\FeedsHTTPFetcher][source]');
  }

  /**
   * Test that nodes will not be created if the user is unauthorized to create
   * them.
   */
  public function testAuthorize() {

    // Create a user with limited permissions.
   $account = $this->drupalCreateUser(array(), 'Development Seed');

    // Adding a mapping to the user_name will invoke authorization.
    $this->addMappings('syndication', array(
      5 => array(
        'source' => 'author_name',
        'target' => 'user_name',
      ),
    ));

    $nid = $this->createFeedNode();

    $this->assertText('Failed importing 10 nodes.');
    $this->assertText('User ' . $account->name . ' is not authorized to create content type article.');
    $node_count = db_query("SELECT COUNT(*) FROM {node}")->fetchField();

    // We should have 1 node, the feed node.
    $this->assertEqual($node_count, 1, t('Correct number of nodes in the database.'));

    // Give the user super admin powers.
    user_delete($account->uid);
    $account = $this->drupalCreateUser(array('access content', 'bypass node access'), 'Development Seed');

    $this->drupalPost("node/$nid/import", array(), 'Import');
    $this->assertText('Created 10 nodes.');
    $node_count = db_query("SELECT COUNT(*) FROM {node}")->fetchField();
    $this->assertEqual($node_count, 11, t('Correct number of nodes in the database.'));
  }

  /**
   * Tests expiring nodes.
   */
  public function testExpiry() {
    // Create importer configuration.
    $this->setSettings('syndication', NULL, array('content_type' => ''));
    $this->setSettings('syndication', 'node', array(
      'expire' => 43200,
    ));

    // Create importer.
    $this->importURL('syndication');

    // Set date of a few nodes to current date so they don't expire.
    $edit = array(
      'date[date]' => date('Y-m-d'),
    );
    $this->drupalPost('node/2/edit', $edit, 'Save and keep published');
    $this->assertText(date('m/d/Y'), 'Found correct date.');
    $this->drupalPost('node/5/edit', $edit, 'Save and keep published');
    $this->assertText(date('m/d/Y'), 'Found correct date.');

    // Run cron to schedule jobs.
    $this->cronRun();

    // Set feeds source expire to run immediately.
    db_update('job_schedule')
      ->fields(array(
        'next' => 0,
      ))
      ->condition('name', 'feeds_source_expire')
      ->execute();

    // Run cron to execute scheduled jobs.
    $this->cronRun();

    // Query the feeds_items table and count the number of entries.
    $row_count = db_query('SELECT COUNT(*) FROM {feeds_item}')->fetchField();

    // Check that number of feeds items is equal to the expected items.
    $this->assertEqual($row_count, 2, 'Nodes expired.');
  }

}
